/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// SyncSettingsAdvanced.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "uiDlg.h"
#include "SyncSettingsAdvanced.h"
#include "pim/ClientSettings.h"
#include "pim/maincpp.h"
#include "tpcshell.h"  // for SHSendBackToFocusWindow
#include <vector>
#include "localizationUtils.h"
#include "processUtils.h"

static int tabStop = 0;
#define CUSTOM_ID_TEXT_LABEL        800

IMPLEMENT_DYNAMIC(CSyncSettingsAdvanced, CDialog)

CSyncSettingsAdvanced::CSyncSettingsAdvanced(CWnd* pParent)
: CDialog(CSyncSettingsAdvanced::IDD, pParent),
      customList(((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl()) {

    // Just to store the local resources handle...
    localResource = getLocalizationUtils()->getLocaleResource();    
    checkEncryption = NULL;
    oldOrientation = -1;
}

CSyncSettingsAdvanced::~CSyncSettingsAdvanced() {
    labels.RemoveAll();
    textFields.RemoveAll();
    labelsDataFormat.RemoveAll();
    mapSources.RemoveAll();
    if (checkEncryption) {
        delete checkEncryption;
    }
    DestroyWindow();  
}

BEGIN_MESSAGE_MAP(CSyncSettingsAdvanced, CDialog)

#ifdef WIN32_PLATFORM_WFSP
    ON_MESSAGE(WM_HOTKEY,&CSyncSettingsAdvanced::OnHotkey)
#endif
    ON_WM_CTLCOLOR()
    ON_WM_VSCROLL()
    ON_WM_SIZE()
#if defined(WIN32_PLATFORM_PSPC)
    ON_BN_SETFOCUS(IDC_ENCRYPTION,          &CSyncSettingsAdvanced::OnBnSetfocusEncryption)
    ON_BN_SETFOCUS(IDC_RADIO_CALENDAR_SIF,  &CSyncSettingsAdvanced::OnBnSetfocusRadioCalendarSif)
    ON_BN_SETFOCUS(IDC_RADIO_CALENDAR_VCARD,&CSyncSettingsAdvanced::OnBnSetfocusRadioCalendarVcard)
    ON_BN_SETFOCUS(IDC_RADIO_TASK_SIF,      &CSyncSettingsAdvanced::OnBnSetfocusRadioTaskSif)
    ON_BN_SETFOCUS(IDC_RADIO_TASK_VCARD,    &CSyncSettingsAdvanced::OnBnSetfocusRadioTaskVcard)    

    ON_EN_SETFOCUS(CUSTOM_ID_TEXT_LABEL,     &CSyncSettingsAdvanced::OnBnSetfocusTextLabel)
    ON_EN_SETFOCUS(CUSTOM_ID_TEXT_LABEL + 1, &CSyncSettingsAdvanced::OnBnSetfocusTextLabel)
    ON_EN_SETFOCUS(CUSTOM_ID_TEXT_LABEL + 2, &CSyncSettingsAdvanced::OnBnSetfocusTextLabel)
    ON_EN_SETFOCUS(CUSTOM_ID_TEXT_LABEL + 3, &CSyncSettingsAdvanced::OnBnSetfocusTextLabel)
    ON_EN_SETFOCUS(CUSTOM_ID_TEXT_LABEL + 4, &CSyncSettingsAdvanced::OnBnSetfocusTextLabel)
    ON_EN_SETFOCUS(CUSTOM_ID_TEXT_LABEL + 5, &CSyncSettingsAdvanced::OnBnSetfocusTextLabel)
    ON_EN_SETFOCUS(CUSTOM_ID_TEXT_LABEL + 6, &CSyncSettingsAdvanced::OnBnSetfocusTextLabel)
    ON_EN_SETFOCUS(CUSTOM_ID_TEXT_LABEL + 7, &CSyncSettingsAdvanced::OnBnSetfocusTextLabel)

#endif
END_MESSAGE_MAP()

void CSyncSettingsAdvanced::DoDataExchange(CDataExchange* pDX) {
    
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_RADIO_CALENDAR_SIF,   radioCalendarSIF);
    DDX_Control(pDX, IDC_RADIO_CALENDAR_VCARD, radioCalendarVCard);
    DDX_Control(pDX, IDC_RADIO_TASK_SIF,       radioTaskSIF);
    DDX_Control(pDX, IDC_RADIO_TASK_VCARD,     radioTaskVCard);

	
}

void CSyncSettingsAdvanced::InitializeFont() { 

    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfHeight += FONT_HEIGHT_OFFSET;
    VERIFY(defaultFontSize.CreateFontIndirect(&lf));
    
    // add the bold
    lf.lfWeight = FW_BOLD;
    VERIFY(fontBold.CreateFontIndirect(&lf));        
}

bool CSyncSettingsAdvanced::createMenu() {   

    // set menubar
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;
    
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
       TRACE(_T("Cannot create command bar!"));
       return false;
    }
    return true;
}

CStatic* CSyncSettingsAdvanced::getLabel(CString label) {
    CStatic *clabel = new CStatic();
    clabel->Create(label, WS_CHILD | WS_VISIBLE, CRect(10, 10, 10, 10), this, 0x188);
    clabel->SetFont(&defaultFontSize);
    return clabel;
}

BOOL CSyncSettingsAdvanced::CreateLabelAndTextBoxButton() {        

    labels.RemoveAll();
    textFields.RemoveAll();   
    labelsDataFormat.RemoveAll();
    mapSources.RemoveAll();
    
    // create dinamically the encryption to make the tabindex right
    checkEncryption = new CButton();
    CRect rect(40, 40, 100, 20); // default values
    checkEncryption->Create(L"Enc", BS_AUTOCHECKBOX | BS_NOTIFY | WS_VISIBLE | WS_TABSTOP, 
                            rect, GetActiveWindow(), IDC_ENCRYPTION);        
    checkEncryption->SetFont(&defaultFontSize);    
    
    for (int i = 0; i < customList.getCountItem(); i++) {
        int id = customList.indexToId(i);
        CString label = customList.getItemName(id);                
        labels.Add(getLabel(label));        
        if (id == SOURCE_CONTACTS || id == SOURCE_CALENDAR ||
            id == SOURCE_TASKS    || id == SOURCE_NOTES) {
                mapSources[id] = getLabel(label);                
        }
        
        CEdit* pEdit = new CEdit();
        pEdit->Create(WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_BORDER, 
                      CRect(20, 20, 100, 40), this, CUSTOM_ID_TEXT_LABEL + i);
        pEdit->SetFont(&defaultFontSize);
        pEdit->EnableWindow(TRUE);
        CString s(customList.getSourceUri(id));
        pEdit->SetWindowTextW(s); 
        pEdit->SetForegroundWindow();
        textFields.Add(pEdit);                          
    }               
    return true;        
}


void CSyncSettingsAdvanced::setStaticString() {
    
    CString s1;
    s1.LoadString(localResource, IDS_ADVANCED_SETTINGS);
    SetWindowText(s1);
    // set ppc header
#if defined(WIN32_PLATFORM_PSPC)
    SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif
    
    s1.LoadString(localResource, IDS_SECURITY);
    SetDlgItemText(IDC_SYNC_STATIC_SECURITY, s1);
    GetDlgItem(IDC_SYNC_STATIC_SECURITY)->SetFont(&fontBold);

    s1.LoadString(localResource, IDS_ENABLE_ENCRYPTION);
    SetDlgItemText(IDC_ENCRYPTION, s1);

    s1.LoadString(localResource, IDS_REMOTENAME);
    SetDlgItemText(IDC_REMOTE_NAMES, s1);
    GetDlgItem(IDC_REMOTE_NAMES)->SetFont(&fontBold);

    // 'Data Format' section
    s1.LoadString(localResource, IDS_DATA_FORMAT);
    SetDlgItemText(IDC_DATA_FORMAT, s1);
    GetDlgItem(IDC_DATA_FORMAT)->SetFont(&fontBold);

    s1.LoadString(localResource, IDS_SETTCALE_FORMAT1); SetDlgItemText(IDC_RADIO_CALENDAR_SIF,   s1);
    s1.LoadString(localResource, IDS_SETTCALE_FORMAT2); SetDlgItemText(IDC_RADIO_CALENDAR_VCARD, s1);
    s1.LoadString(localResource, IDS_SETTTASK_FORMAT1); SetDlgItemText(IDC_RADIO_TASK_SIF,       s1);
    s1.LoadString(localResource, IDS_SETTTASK_FORMAT2); SetDlgItemText(IDC_RADIO_TASK_VCARD,     s1);
}

void CSyncSettingsAdvanced::setScrollSize(int last_y) {
    
    //
    // Manage the vertical scrolling-bar
    //
    //GetWindowRect(&dlgRect);
    GetClientRect(&dlgRect);
    scrollbarPosition = 0;                   // reset bar position
    
    dlgMaxHeight = (last_y - dlgRect.top); 
    //dlgMaxHeight += 52; // offset value
    //dlgMaxHeight += (int)(dlgMaxHeight * 0.14);

    // Set all scrollbar parameters    
    si.cbSize = sizeof(SCROLLINFO);
    si.fMask = SIF_ALL;                 // SIF_ALL = SIF_RANGE | SIF_PAGE | SIF_POS | SIF_TRACKPOS;
    si.nMin = 0;
    si.nMax = dlgMaxHeight;
    si.nPage = dlgRect.Height();
    //si.nPos = 0;
    //SetScrollInfo(SB_VERT, &si, TRUE);
    //int val = (int)(si.nMax / si.nPage);

    // the position is relative to window Rect that was created before
    int pos = ::GetScrollPos(GetSafeHwnd(), SB_VERT);   
    if (pos != 0) {
        // scrollbarPosition = (int)((pos * (dlgRect.bottom)) / (previousBottomOfWinRec));        
        // scrollbarPosition = (int)(pos * val / (int)(si.nMax / si.nPage));
        scrollbarPosition = pos;
        si.nPos = scrollbarPosition;        
    } else {
        scrollbarPosition = 0;
        si.nPos = 0;
    }
    previousBottomOfWinRec = dlgRect.bottom;
    SetScrollInfo(SB_VERT, &si, TRUE);
    
    if ((pos = ::GetScrollPos(GetSafeHwnd(), SB_VERT)) != scrollbarPosition) {
        ScrollWindowEx(0, (scrollbarPosition-pos), NULL, &dlgRect, NULL, NULL, SW_SCROLLCHILDREN | SW_INVALIDATE);
        scrollbarPosition = pos;
        //si.nPos = scrollbarPosition; 
        //SetScrollInfo(SB_VERT, &si, TRUE);
    }
}

void CSyncSettingsAdvanced::loadRadioValues() {
    for (int i = 0; i < customList.getCountItem(); i++) {
        int id = customList.indexToId(i); 
        bool ret = false;
        if (id == SOURCE_CALENDAR) {
            ret = customList.getUseSif(id);
            if (ret) {
                radioCalendarSIF.SetCheck(BST_CHECKED);
                radioCalendarVCard.SetCheck(BST_UNCHECKED);
            } else {
                radioCalendarVCard.SetCheck(BST_CHECKED);
                radioCalendarSIF.SetCheck(BST_UNCHECKED);
            }            
        } else if (id == SOURCE_TASKS) {
            ret = customList.getUseSif(id);
            if (ret) {
                radioTaskSIF.SetCheck(BST_CHECKED);
                radioTaskVCard.SetCheck(BST_UNCHECKED);
            } else {
                radioTaskVCard.SetCheck(BST_CHECKED);
                radioTaskSIF.SetCheck(BST_UNCHECKED);
            }
        }                                           
    }       
}

void CSyncSettingsAdvanced::setRadioValues() {
        
    for (int i = 0; i < customList.getCountItem(); i++) {
        int id = customList.indexToId(i);         
        if (id == SOURCE_CALENDAR) {
            if(radioCalendarSIF.GetCheck() == BST_CHECKED) {
                customList.setUseSif(id, true);
            } else {
                customList.setUseSif(id, false);
            }           
        } else if (id == SOURCE_TASKS) {
            if(radioTaskSIF.GetCheck() == BST_CHECKED) {
                customList.setUseSif(id, true);
            } else {
                customList.setUseSif(id, false);
            }
        }                                           
    }       
}

void CSyncSettingsAdvanced::loadEncryptionValue() {
    bool ret = false;    
    for (int i = 0; i < customList.getCountItem(); i++) {
        int id = customList.indexToId(i); 
        ret = customList.getUseEncryption(id);   
        break;
    }
    if(ret && checkEncryption) {
        checkEncryption->SetCheck(BST_CHECKED);
    } else {
        checkEncryption->SetCheck(BST_UNCHECKED);
    }
}

void CSyncSettingsAdvanced::setEncryptionValue(bool val) {

    for (int i = 0; i < customList.getCountItem(); i++) {
        int id = customList.indexToId(i); 
        customList.setUseEncryption(id, val);           
    }
}

BOOL CSyncSettingsAdvanced::OnInitDialog() {
            
    CDialog::OnInitDialog();
    
    InitializeFont();
    if (!createMenu()) {
        return false;
    }
    
    // To start, hide all the commands that could be not visible
    GetDlgItem(IDC_ADV_CONTACTS_TYPE_LABEL)->ShowWindow(SW_HIDE);
    GetDlgItem(IDC_RADIO_CALENDAR_SIF)->ShowWindow(SW_HIDE);
    GetDlgItem(IDC_RADIO_CALENDAR_VCARD)->ShowWindow(SW_HIDE);
    GetDlgItem(IDC_RADIO_TASK_SIF)->ShowWindow(SW_HIDE);
    GetDlgItem(IDC_RADIO_TASK_VCARD)->ShowWindow(SW_HIDE);
    GetDlgItem(IDC_SYNC_NOTE_BOX2)->ShowWindow(SW_HIDE);
    GetDlgItem(IDC_DATA_FORMAT)->ShowWindow(SW_HIDE);    

    //
    // Create the label and text box populated with the right source uri
    //
    CreateLabelAndTextBoxButton();
    
    loadRadioValues();
    loadEncryptionValue();
    setStaticString();
    
    // initilize the position of the static labels. These positions are used
    // for the repaint of the scroll bar
#if defined(WIN32_PLATFORM_PSPC)    
    GetDlgItem(IDC_HEADER_STATIC)->GetWindowRect(&labelStaticHeader);    
    GetDlgItem(IDC_HEADER_STATIC)->GetClientRect(&clientLabelStaticHeader);       
#endif    
    GetDlgItem(IDC_SYNC_STATIC_SECURITY)->GetWindowRect(&labelSecurityOrig);    
    GetDlgItem(IDC_SYNC_STATIC_SECURITY)->GetClientRect(&clientLabelSecurityOrig);
    GetDlgItem(IDC_REMOTE_NAMES)->GetWindowRect(&labelRecOrig);    
    GetDlgItem(IDC_REMOTE_NAMES)->GetClientRect(&clientLabelRecOrig);

#ifdef WIN32_PLATFORM_WFSP
    // override back key
    LPARAM lparam = MAKELPARAM(SHMBOF_NODEFAULT | SHMBOF_NOTIFY,
        SHMBOF_NODEFAULT | SHMBOF_NOTIFY);
    ::SendMessage(dlgCommandBar.hwndMB, SHCMBM_OVERRIDEKEY, VK_TBACK, lparam);
#endif
    
    return FALSE;
}


/**
 * Save settings to registry
 */
afx_msg void CSyncSettingsAdvanced::OnOK(){

    ClientSettings* cs = getRegConfig();
    
    if(checkStartSync()){
        CDialog::OnOK();
    }
    if (checkEncryption) {
        if(checkEncryption->GetCheck() == BST_CHECKED) {        
            setEncryptionValue(true);
        } else {
            setEncryptionValue(false);
        }
    }

    int id = -1;
    CEdit* pEdit = NULL;
    CString fromText, fromSource;
    for (int i = 0; i < customList.getCountItem(); i++) {
        
        pEdit = textFields.GetAt(i);
        pEdit->GetWindowTextW(fromText);
        id = customList.indexToId(i);             
        wstring s((LPCTSTR)fromText); 
        char* tmp = toMultibyte(s.c_str());                
        customList.setSourceUri(id, tmp);
        delete [] tmp;        
    }
    
    setRadioValues();

    // Read current selections and initial selections    
    getRegConfig()->setClientDirty(CS_DIRTY_SOURCE_ALL);
    getRegConfig()->setClientDirty(CS_DIRTY_ROOT_CONFIG);
    getRegConfig()->saveDirty();    
   
    CDialog::OnOK();
}


// CSyncSettingsAdvanced message handlers
HBRUSH CSyncSettingsAdvanced::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
    #endif

    return hbr;
}


/**
 * Called when vertical scrollbar is touched.
 * Manages the window scrolling.
 */
void CSyncSettingsAdvanced::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)  {
    int nDelta;
    int nMaxPos = dlgMaxHeight;

    // Calculate the offset to scroll (nDelta)
    switch (nSBCode) {
        case SB_LINEDOWN:
            if (scrollbarPosition >= nMaxPos)
                return;
            nDelta = 20;
            break;

        case SB_LINEUP:
            if (scrollbarPosition <= 0)
                return;
            nDelta = -20;
            break;

        case SB_THUMBPOSITION:
            nDelta = (int)nPos - scrollbarPosition;
            break;

        case SB_PAGEDOWN:        
            if (scrollbarPosition >= nMaxPos)
                return;            
            nDelta = min(nMaxPos/10, nMaxPos - scrollbarPosition);
            break;                           
        case SB_PAGEUP:
            if (scrollbarPosition <= 0) {
                return;
            }
            nDelta = -min(nMaxPos/10, scrollbarPosition);
            break;
         case SB_THUMBTRACK:                
            if (LOBYTE(nSBCode) == SB_THUMBTRACK) {                           
                nDelta = nPos - scrollbarPosition;
            }
            break;

        default:
            // Don't handle other commands...
            return;
    }

    // Move the scrollbar
    scrollbarPosition += nDelta;
    SetScrollPos(SB_VERT, scrollbarPosition, TRUE);

    // Scroll the dialog window
    ScrollWindowEx(0, -nDelta, NULL, &dlgRect, NULL, NULL, SW_SCROLLCHILDREN | SW_INVALIDATE);
    
    CDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

/**
 * Called when window size changes. We have to redefine the scrollbar
 * parameters.
 */
void CSyncSettingsAdvanced::OnSize(UINT nType, int cx, int cy) {
    
    CDialog::OnSize(nType, cx, cy);    
    int resId = IDD_UI_SYNC_ADVANCED;
    if (oldOrientation == -1) {
        DRA::RelayoutDialog(AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));                
        oldOrientation = -2; // just a value that is overwritten after
    }
    
    if (labels.GetCount() == 0) {
        return;
    }

    CString s1 ,s2;
    int y_offset_from_label = 0;
    int y_offset = 0;
    int y_offset_from_last_source = 0;
    int y_label_size = 18;  // fixed size of the label
    int y_text_size  = 18;  // fixed size of the text
    int x_radio_size_first = 45;  // first radio button
    int x_radio_size_second = 75; // second radio button: containt long string
    
    int moltiplicator = 1;
    if (CuiDlg::isVGA()) {
        moltiplicator = 2;
    }
    
    switch(DRA::GetDisplayMode()) {        
        case DRA::Landscape: 
            {
                newOrientation = 0;
                y_offset_from_label = 5*moltiplicator; // distance between the first label                                        
                y_offset = 2*moltiplicator;            // distance between the checkbox
                y_offset_from_last_source = 5*moltiplicator;                 
            }
            break;
        
        case DRA::Square:
            {
                newOrientation = 1;
                y_offset_from_label = 3*moltiplicator; // distance between the first label                                        
                y_offset = 2*moltiplicator;            // distance between the checkbox  
                y_offset_from_last_source = 5*moltiplicator; 
#if defined(WIN32_PLATFORM_PSPC)
    if (CuiDlg::isQVGASquare()) {
        y_label_size = y_label_size + 4;  
        y_text_size  = y_text_size + 4;  
        x_radio_size_first = 45 + 10;  
        x_radio_size_second = 75 + 20; 
   
    }
#endif
            }
            break;
        default: //  also case DRA::Portrait:      
        {       
            newOrientation = 2;
            y_offset_from_label = 5*moltiplicator; // distance between the first label                                        
            y_offset = 4*moltiplicator;            // distance between the checkbox  
            y_offset_from_last_source = 5*moltiplicator; 
        }    
        break;
    }        
    
    if (oldOrientation != newOrientation) {
        oldOrientation = newOrientation;
        changed = true;
        // reset the scrollbar
        scrollbarPosition = 0;
        SetScrollPos(SB_VERT, scrollbarPosition, TRUE);        
        
    }

    CRect clientRect;  GetClientRect(&clientRect);    
    CRect winRect;     GetWindowRect(&winRect);

    // normalized fixed size
    y_label_size *= moltiplicator;
    y_text_size  *= moltiplicator;   
    x_radio_size_first *= moltiplicator;  // first radio button
    x_radio_size_second *= moltiplicator; // second radio button: containt long string
   

    // difference between top of windowRect and Client rect. This is used in the
    // SetWindowsPos
    int offset = winRect.BottomRight().y - clientRect.BottomRight().y;        

    if (changed) {
        // repaint the label starting from the original values
#if defined(WIN32_PLATFORM_PSPC) 
        GetDlgItem(IDC_HEADER_STATIC)->SetWindowPos(&CWnd::wndTop,
            labelStaticHeader.left, 
            labelStaticHeader.top - offset,                 
            clientLabelStaticHeader.right, 
            clientLabelStaticHeader.bottom, 
            SWP_NOZORDER);         

#endif
        // position of the first fixed label
        GetDlgItem(IDC_REMOTE_NAMES)->SetWindowPos(&CWnd::wndTop,
            labelRecOrig.left, 
            labelRecOrig.top - offset,                 
            clientLabelRecOrig.right, 
            clientLabelRecOrig.bottom, 
            SWP_NOZORDER);       
        // position of the first fixed label
        GetDlgItem(IDC_SYNC_STATIC_SECURITY)->SetWindowPos(&CWnd::wndTop,
            labelSecurityOrig.left, 
            labelSecurityOrig.top - offset,                 
            clientLabelSecurityOrig.right, 
            clientLabelSecurityOrig.bottom, 
            SWP_NOZORDER);        
    }

    CRect labelRec, staticRec;    
    GetDlgItem(IDC_REMOTE_NAMES)->GetWindowRect(&labelRec);    
    
    CWnd* pWnd = NULL;    
    
    // start info and size for the first items to be drawn
    // now draw the labels and the text box for every source
    int x_label_start     = (int)(winRect.BottomRight().x * 0.04);  // 4% of the lenght of the screen. 
    int y_label_start     = labelRec.BottomRight().y - offset + y_offset_from_label;
    int x_label_size      = (int)(winRect.BottomRight().x * 0.25); // 25% of lenght of screen 
    // int y_label_size      = 18;  // see above
    
    int x_text_start      = (int)(winRect.BottomRight().x * 0.32);  // 32% of the lenght of the screen. 
    int y_text_start      = labelRec.BottomRight().y - offset + y_offset_from_label;
    int x_text_size       = (int)(winRect.BottomRight().x * 0.55); // 55% of lenght of screen 
    // int y_text_size       = 18;  // see above
    
     // position of the encryption checkbox based on the remote Name label as the other
    CRect securityRect;     
    GetDlgItem(IDC_SYNC_STATIC_SECURITY)->GetWindowRect(&securityRect);    
    pWnd = GetDlgItem(IDC_ENCRYPTION);
    if (pWnd) {
        pWnd->SetWindowPos(GetActiveWindow(), x_text_start,
                                            securityRect.TopLeft().y - offset,
                                            x_text_size,
                                            y_text_size,
                                            SWP_NOZORDER);
        pWnd->ShowWindow(SW_SHOW);

    }

    for (int i = 0; i < customList.getCountItem(); i++) {                    
        
        if (labels.GetCount() > 0) {
            CStatic* cs = labels.GetAt(i);
            CEdit*   ce = textFields.GetAt(i);
            if (cs) {  
                cs->SetWindowPos(GetActiveWindow(), x_label_start,
                    y_label_start,
                    x_label_size,
                    y_label_size,
                    SWP_NOZORDER);
                y_label_start += (y_label_size + y_offset);


                ce->SetWindowPos(GetActiveWindow(), x_text_start,
                    y_text_start,
                    x_text_size,
                    y_text_size,
                    SWP_NOZORDER);
                y_text_start += (y_text_size + y_offset);                        

            } else {
                break;
            }
        }
    }
 
    
    // set the position of the data format label
    pWnd = GetDlgItem(IDC_DATA_FORMAT);
    y_label_start += y_offset_from_last_source;
    if (pWnd) {
        pWnd->SetWindowPos(GetActiveWindow(), labelRec.left,
                    y_label_start,
                    x_text_size,
                    y_text_size,
                    SWP_NOZORDER);
    
    }
    y_label_start += (y_label_size + y_offset);
    y_text_start = y_label_start; // align the y coordinate

    // it is a fixed size of the radio. Used both for the size of the windows and to 
    // distantiate the radio buttons
    //int x_radio_size_first = 45;  // see above
    //int x_radio_size_second = 75; // see above
    //
    // Visualize now only the sources contact, calendar, task and note. So
    // check if they are in the array list or not     
    //  
    for (int i = 0; i < customList.getCountItem() && mapSources.GetSize() > 0; i++) { 
        int id = customList.indexToId(i);
        CStatic* cs = mapSources[id];
        if (cs) {
            GetDlgItem(IDC_DATA_FORMAT)->ShowWindow(SW_SHOW); // enable the label
            cs->SetWindowPos(GetActiveWindow(), x_label_start,
                y_label_start,
                x_label_size,
                y_label_size,
                SWP_NOZORDER);
            y_label_start += (y_label_size + y_offset); 

            switch(id) {
            case SOURCE_CONTACTS:
            case SOURCE_NOTES:
                {             
                    if (id == SOURCE_CONTACTS) {
                        pWnd = GetDlgItem(IDC_ADV_CONTACTS_TYPE_LABEL);
                        GetDlgItem(IDC_ADV_CONTACTS_TYPE_LABEL)->ShowWindow(SW_SHOW);
                    } else {
                        pWnd = GetDlgItem(IDC_SYNC_NOTE_BOX2);
                        GetDlgItem(IDC_SYNC_NOTE_BOX2)->ShowWindow(SW_SHOW);                        
                    }
                    
                    pWnd->SetWindowPos(GetActiveWindow(), x_text_start,
                        y_text_start,
                        x_text_size,
                        y_text_size,
                        SWP_NOZORDER);  
                    y_text_start += (y_text_size + y_offset);

                }
                break;
            case SOURCE_CALENDAR:
            case SOURCE_TASKS:
                {     
                    CWnd* pRadio = NULL;
                    if (id == SOURCE_CALENDAR) {
                        pWnd   = GetDlgItem(IDC_RADIO_CALENDAR_SIF);
                        pRadio = GetDlgItem(IDC_RADIO_CALENDAR_VCARD);
                        GetDlgItem(IDC_RADIO_CALENDAR_SIF)->ShowWindow(SW_SHOW);
                        GetDlgItem(IDC_RADIO_CALENDAR_VCARD)->ShowWindow(SW_SHOW);                        
                         
                    } else {
                        pWnd = GetDlgItem(IDC_RADIO_TASK_SIF);
                        pRadio = GetDlgItem(IDC_RADIO_TASK_VCARD);
                        GetDlgItem(IDC_RADIO_TASK_SIF)->ShowWindow(SW_SHOW);
                        GetDlgItem(IDC_RADIO_TASK_VCARD)->ShowWindow(SW_SHOW);
                    }
                    // draw the radio buttons
                    pWnd->SetWindowPos(GetActiveWindow(), x_text_start,
                        y_text_start,
                        x_radio_size_first,
                        y_text_size,
                        SWP_NOZORDER);  

                    pRadio->SetWindowPos(GetActiveWindow(), x_text_start + x_radio_size_first,
                        y_text_start,
                        x_radio_size_second,
                        y_text_size,
                        SWP_NOZORDER);  
                    y_text_start += (y_text_size + y_offset);
                }
                break;                    
            default: 
                {
                    break;
                }
            }
        }              
    }  
    

    if (y_text_start > winRect.bottom - offset) {          
        if (changed) {     
            initScroll = y_text_start;
            previousBottomOfWinRec = clientRect.bottom;
            // previousBottomOfWinRec = winRect.bottom;
            scrollBarExists = true;
            changed = false;            
            ::ShowScrollBar(GetSafeHwnd(), SB_VERT, true);        
        }
        setScrollSize(initScroll);        
    } else {
        if (changed) {            
            scrollBarExists = false;
            changed = false;
            ::ShowScrollBar(GetSafeHwnd(), SB_VERT, false);        
        }
        if (scrollBarExists) {
            setScrollSize(initScroll);            
        }

    }
        
    if (checkEncryption) {
        checkEncryption->SetFocus();
    }
#if defined(WIN32_PLATFORM_WFSP)
    ::SendMessage(GetSafeHwnd(), DM_RESETSCROLL, TRUE, TRUE);
#endif    

  
}

#ifdef WIN32_PLATFORM_WFSP
LRESULT CSyncSettingsAdvanced::OnHotkey(WPARAM wParam, LPARAM lParam) {

    if (HIWORD(lParam) == VK_TBACK)
        SHSendBackToFocusWindow(WM_HOTKEY, wParam, lParam);
    return 0;
}
#endif

#if defined(WIN32_PLATFORM_PSPC)
void CSyncSettingsAdvanced::setScrollOnFocus(int objectIDC){
    
    //
    // If the scrollbar doesn't exist nothing to do...
    //
    if (scrollBarExists == false) {
        return;
    }
    CRect dRect;
    CRect textRect;

    GetWindowRect(&dRect);
    GetDlgItem(objectIDC)->GetWindowRect(&textRect);

    int delta = textRect.bottom - (dRect.Height() / 2);

    int oldScrollbarPosition = scrollbarPosition;
    scrollbarPosition += delta;
    
    if (scrollbarPosition<0){
        scrollbarPosition = 0;
    }
    if (scrollbarPosition > dlgMaxHeight - dRect.Height()){
        scrollbarPosition = dlgMaxHeight - dRect.Height();
    }

    delta = scrollbarPosition - oldScrollbarPosition;
    
    // Move the scrollbar
    SetScrollPos(SB_VERT, scrollbarPosition, TRUE);

    // Scroll the dialog window
    ScrollWindowEx(0, -delta, NULL, &dRect, NULL, NULL, SW_SCROLLCHILDREN | SW_INVALIDATE);
}


void CSyncSettingsAdvanced::OnBnSetfocusEncryption() {
    setScrollOnFocus(IDC_ENCRYPTION);
}

void CSyncSettingsAdvanced::OnBnSetfocusRadioCalendarSif() {
     
    setScrollOnFocus(IDC_RADIO_CALENDAR_SIF);
}

void CSyncSettingsAdvanced::OnBnSetfocusRadioCalendarVcard() {
    setScrollOnFocus(IDC_RADIO_CALENDAR_VCARD);
}

void CSyncSettingsAdvanced::OnBnSetfocusRadioTaskSif() {
    setScrollOnFocus(IDC_RADIO_TASK_SIF);
}

void CSyncSettingsAdvanced::OnBnSetfocusRadioTaskVcard() {
    setScrollOnFocus(IDC_RADIO_TASK_VCARD);
}


void CSyncSettingsAdvanced::OnBnSetfocusTextLabel() {    

    HWND wndFocus = GetFocus()->GetSafeHwnd();
    
    for (int i = 0; i < textFields.GetCount(); i++) {
        try {
            if (wndFocus == GetDlgItem(CUSTOM_ID_TEXT_LABEL + i)->GetSafeHwnd()) {
                setScrollOnFocus(CUSTOM_ID_TEXT_LABEL + i);
                return;
            } 

        } catch (...) {

        }
    }
}

#endif