/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef INCL_MAIL_CLIENT2SERVER
#define INCL_MAIL_CLIENT2SERVER

#include "base/fscapi.h"
#include "base/util/ArrayList.h"
#include "base/util/ArrayElement.h"
#include "pim/mail/MailInfo.h"
#include "pim/utils.h"
#include "pim/mail/MailUtils.h"
#include "spds/constants.h"
#include "spds/SyncItem.h"
#include "mail/MailMessage.h"
#include "pim/mail/MailClientData.h"

#include "spds/constants.h"

#include <mapidefs.h>
#include <mapicode.h>
#include <mapitags.h>
#include <mapix.h>
#include <mapiutil.h>
#include <cemapi.h>

class MailClient2Server {

private:
    /**
     * Avoid object copy
     */
    MailClient2Server(MailClient2Server& o);

    MailClient2Server& operator=(MailClient2Server& o);

public:

    /**
     * Constructor: create a Container
     */
    MailClient2Server();

    /**
     * Destructor:
     */
    ~MailClient2Server();

    /**
     * Function to complete the object to sync. It returns a complete object representing and email.
     * The isUpdate parameter says the mailClientData to fill is an updated item. In this case it full
     * completes the object only if it is Draft folder else it fills only the syncml email header info
     * (currently the LastModificationTime and the read parameter)
     */
    int completeObject(MailClientData& mcd, IMessage* pmsg, const bool isUpdate);

    HRESULT GetRecipients(IMessage* pmsg, char** to, char** cc, char** bcc);
    HRESULT GetBody(IMessage* pmsg, wchar_t** body);
    HRESULT GetAttachments(IMessage* pmsg, MailClientData* mcd);

    /**
     * Sets the message's flag read to true.
     */
    HRESULT markAsRead(IMessage* pMsg);

    /**
     * Returns a new allocated MailClientData ready to be sent.
     * @param content  the content to be filled inside the message body
     * @return new allocated MailClientData*, NULL in case of errors
     */
    MailClientData* createMailForSendLog(const StringBuffer& content);
};

#endif
