/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#include "base/Log.h"
#include "base/util/utils.h"
#include "base/startcmd.h"
#include "SyncMLTransport.h"
#include <Windows.h>
#include "aygshell.h"
#include "resource.h"
#include "notify/timed_msgbox.h"
#include "localizationUtils.h"

#define USES_IID_IMAPIAdviseSink
#include <cemapi.h>
#include "base/Log.h"
#include "processUtils.h"
#include "pim/mail/MailUtils.h"
#include "pim/mail/MessagingManager.h"
#include "pim/mail/MessagingStatus.h"

#include "http/TransportAgentFactory.h"
#include "http/TransportAgent.h"
#include "http/Proxy.h"
#include "http/URL.h"

#include "AdviseSink.h"

bool fromFlagUpdate;
IMAPISession *    globalpSession;
IMessage *        globalpmsg    ;

CAdviseSink::CAdviseSink()
{
    LOG.debug("CAdviseSink created");
    globalpSession    =  NULL;
    globalpmsg        =  NULL;

    mssc   = getRegConfig()->getMailSyncSourceConfig(true);
    msgMan = getMessagingManager();
};

CAdviseSink::~CAdviseSink()
{
    LOG.debug("CAdviseSink destroyed");
};


STDMETHODIMP CAdviseSink::QueryInterface(REFIID riid,  LPVOID * ppvObj) {

    LOG.debug("CAdviseSink QueryInterface");

    *ppvObj = 0;

    *ppvObj = (LPVOID)this;
    AddRef();
    return S_OK;

    return E_NOINTERFACE;

};

STDMETHODIMP_(ULONG) CAdviseSink::AddRef()
{
    cRef++;
    return cRef;
};

STDMETHODIMP_(ULONG) CAdviseSink::Release()
{
    if (cRef > 0) --cRef;
    return cRef;
};

enum {
    ePR_ENTRYID,
    ePR_MSG_STATUS,
    eNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eNUM_COLS, Columns) =
{
    eNUM_COLS,
    PR_ENTRYID,
    PR_MSG_STATUS
};

//------------------------------------------------------------- Typedef

enum {
    eeePR_ENTRYID,
    ePR_MESSAGE_FLAGS,
    //MM
    ePR_LAST_MODIFICATION_TIME,
    ePR_SUBJECT,
    ePR_SUBJECT_PREFIX,
    ePR_MESSAGE_SIZE,
    ePR_SENDER_NAME,
    ePR_SENDER_EMAIL_ADDRESS,
    ePR_HASATTACH,
    ePR_MESSAGE_DELIVERY_TIME,
    ePR_CUSTOM_MESSAGE_ID,
    ePR_CUSTOM_MESSAGE_HEADER,
    ePR_REPLY_RECIPIENT_NAMES,
    ePR_REPLY_RECIPIENT_ENTRIES,
    ePR_CLIENT_SUBMIT_TIME,
    ePR_IMPORTANCE,
    //MM
    NUM_COLS};

static SizedSPropTagArray(NUM_COLS, folderRoleProperty) =
{
    NUM_COLS,
    PR_CE_IPM_INBOX_ENTRYID
};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(NUM_COLS, properties) =
{
    NUM_COLS,
    PR_ENTRYID,                         // entry id
    PR_MESSAGE_FLAGS,
    //MM
    PR_LAST_MODIFICATION_TIME,          //
    PR_SUBJECT,                         // mail Subject
    PR_SUBJECT_PREFIX,
    PR_MESSAGE_SIZE,                    // sum of the size of every property of the object
    PR_SENDER_NAME,                     // "Marco "
    PR_SENDER_EMAIL_ADDRESS,            // From  "Marco <magi@libero.it>
    PR_HASATTACH,
    PR_MESSAGE_DELIVERY_TIME,
    PR_CUSTOM_MESSAGE_ID,
    PR_CUSTOM_MESSAGE_HEADER,
    PR_REPLY_RECIPIENT_NAMES,
    PR_REPLY_RECIPIENT_ENTRIES,
    PR_CLIENT_SUBMIT_TIME,
    PR_IMPORTANCE  

};

const wstring CAdviseSink::getMailEntryId(SBinary sbEntry) {
    
    HRESULT         hr          = S_OK;
    LPUNKNOWN*      object      = NULL;
	IMAPIFolder*    folder      = NULL;
	IMessage*       pmsg		= NULL;
    IMAPISession*   pSession    =  NULL;
    ULONG           returned    =  0;
    ULONG           rgMsgTags[] = { 1, PR_ENTRYID };
    ULONG           cmsgValue   = 0;
    SPropValue*     msgprops    = NULL;
    wstring msgId;
   
    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);

    hr = pSession->OpenEntry(sbEntry.cb,
        (ENTRYID*)sbEntry.lpb,
        NULL,
        MAPI_BEST_ACCESS,
        &returned,
        (IUnknown **) &pmsg);

    if (!pmsg){
        LOG.error("%s: pmsg is null", __FUNCTION__);
        goto FuncExit;
    }
 
    hr = pmsg->GetProps ((LPSPropTagArray)&properties, MAPI_UNICODE, &cmsgValue, &msgprops);
    EXIT_ON_FAILED (hr);

    if ( !msgprops ) {
        LOG.error("%s: can't get message props.", __FUNCTION__);
        goto FuncExit;
    }

    if (msgprops[ePR_ENTRYID].ulPropTag == PR_ENTRYID) {
        msgId = MessagingManager::createKeyFromId(msgprops[ePR_ENTRYID].Value.bin);
    } else {
        LOG.error("error getting msg entry id.");
    }

FuncExit:
    if (msgprops) {
        MAPIFreeBuffer (msgprops);
    }

    RELEASE_OBJ(pSession);
    RELEASE_OBJ (pmsg);

    return msgId;
}


// This method uses the Sbinary of the mail and its parent to 
// create an item data to add to the messaging status that will
// write the changes to the internal cache.
ItemData* CAdviseSink::getItemData(SBinary sbEntry, const wstring& parentKey) 
{
    wstring id = getMailEntryId(sbEntry);
    ItemData* item = new ItemData(id, MAPI_MESSAGE);

    item->setParentId(parentKey);
    
    return item;
}

int CAdviseSink::isMailDeleted(SBinary sbEntry, const wstring& parentKey)
{
    int ret = 0;
    ItemData* item = getItemData(sbEntry, parentKey);
    item->setStatus(item_status_deleted);
    item->setParentRole(ROLE_INBOX);
    if (getMessagingStatus()->updateMessagingStatusItem(*item)) {
        ret = 0;
    } else {
        ret = 1;
    }

    delete item;
    LOG.debug("AdviseSink: Mail Deleted");

    return ret;
}

int CAdviseSink::isNewMailSent(SBinary sbEntry, const wstring& parentKey)
{
    int ret = 0;

    ItemData* item = getItemData(sbEntry, parentKey);
    item->setStatus(item_status_new);
    item->setParentRole(ROLE_OUTBOX);

    if (getMessagingStatus()->addMessagingStatusItem(*item)){
        ret = 0;
    } else {
        ret = 1;
    }
    delete item;

    LOG.debug("AdviseSink: new mail sent");
    return ret;
}


int CAdviseSink::isMailMoved(SBinary sbEntry, const wstring& parentKey)
{
    int ret = 0;
    
    LOG.debug("%s", __FUNCTION__);

    return ret;
}

int CAdviseSink::isMarkedChangedFlag(SBinary sbEntry, 
                                     const wstring& parentKey,
                                     const StringBuffer& parentRole)
{
    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    ULONG             returned    =  0;
    IMessage *        pmsg        =  NULL;
    ULONG             messageStatus = (ULONG) 0x00000000;
    int ret = 0;

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);

    hr = pSession->OpenEntry(sbEntry.cb,
        (ENTRYID*)sbEntry.lpb,
        NULL,
        MAPI_BEST_ACCESS,
        &returned,
        (IUnknown **) &pmsg);

    if (!pmsg){
        LOG.error("updateFlagMail: pmsg is null.");
        goto FuncExit;
    }
 
    SPropValue* msgprops = NULL;
    ULONG    cmsgValue =  0;

    hr = pmsg->GetProps ((LPSPropTagArray)&properties, MAPI_UNICODE, &cmsgValue,
       &msgprops);
    EXIT_ON_FAILED (hr);

    if (!msgprops) {
        LOG.error("can't get message props.");
        goto FuncExit;
    }

    bool read = false;
    
    if (msgprops[ePR_MESSAGE_FLAGS].ulPropTag == PR_MESSAGE_FLAGS) {    
        ItemData* itemMoved = getItemData(sbEntry, parentKey);
        itemMoved->setStatus(item_status_updated);
       
        if (msgprops[ePR_MESSAGE_FLAGS].Value.ul & MSGFLAG_READ) {
            LOG.debug("AdviseSink: setted flag to read");
            read = true;
        }

        itemMoved->setReadFlag(read);
        itemMoved->setParentRole(parentRole);

        if (getMessagingStatus()->updateMessagingStatusItem(*itemMoved)){
            ret = 0;
        } else {
            ret = 1;
        }
        delete itemMoved;
    }
    
FuncExit:
    
    if (msgprops) {
        MAPIFreeBuffer (msgprops);
    }
    if (pmsg) {
        pmsg->Release();
    }
    if (pSession) {
        pSession->Release();
    }

    return ret;
}


/*
* to understand if the message is marked to downlaod
* it returns 1 if the body link is selected
* it returns 2 if the attachment link is selected. 
* it returns 0 if no status is the needed
*/

int CAdviseSink::isMarkedToDownload(SBinary sbEntry) {

    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    ULONG             returned    =  0;
    IMessage *        pmsg        =  NULL;
    ULONG             messageStatus = (ULONG) 0x00000000;
    int ret = 0;

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);

    hr = pSession->OpenEntry(sbEntry.cb,
        (ENTRYID*)sbEntry.lpb,
        NULL,
        MAPI_BEST_ACCESS,
        &returned,
        (IUnknown **) &pmsg);

    if(!pmsg){
        LOG.error("updateFlagMail: pmsg is null.");
        goto FuncExit;
    }

    SPropValue* msgprops = NULL;
    ULONG    cmsgValue =  0;

    hr = pmsg->GetProps ((LPSPropTagArray)&Columns, MAPI_UNICODE, &cmsgValue,
       &msgprops);

    if (msgprops == NULL) {
        LOG.error("error getting message properties");

        goto FuncExit;
    }
    
    if (msgprops[ePR_MSG_STATUS].ulPropTag == PR_MSG_STATUS) {
        messageStatus = msgprops[ePR_MSG_STATUS].Value.ul;
        if (messageStatus & MSGSTATUS_REMOTE_DOWNLOAD)
            ret = 1;
        // for attachments:
        else if (messageStatus & MSGSTATUS_REMOTE_DOWNLOAD_ATTACH) { 
            // set the info in the attachmentParams object
            getAttachmentParams(pmsg);            
            ret = 2; 
        } 
    }

    MAPIFreeBuffer (msgprops);

FuncExit:
    if (pmsg) {
        pmsg->Release();
    }

    if (pSession) {
        pSession->Release();
    }
    return ret;
}

bool CAdviseSink::getAttachmentParams(IMessage* pmsg) {

    LPMAPITABLE pAttachTable = NULL;
    LPSRowSet pRows = NULL;
    int currentRow = 0;
    char buffer[2048];
    ULONG ulNumChars;
    bool ret = false;
    HRESULT hr = pmsg->GetAttachmentTable(0, &pAttachTable);
    EXIT_ON_FAILED(hr);

    while(TRUE) {

        hr = pAttachTable->QueryRows(1, 0, &pRows);
        if (FAILED(hr) || pRows->cRows != 1)
            break;
        currentRow++;
        
        if (ret) {
            continue;
        }
        // only one row is returned anyway
        int nProps = pRows->aRow[0].cValues;
        long num = -1;
        long size = -1;
        wstring filename;
        long msgstatus = -1;
        IAttach* attach;

        for (int i = 0; i < nProps; i++) {
            ULONG ulPT = pRows->aRow[0].lpProps[i].ulPropTag;
            // loop through all the properties

            // TCHAR* szPropName = ReverseLookup(ulPT);
            switch (ulPT) {
                case PR_ATTACH_NUM:
                    num = pRows->aRow[0].lpProps[i].Value.ul;
                    attachmentParam.setNumber(num);
                    break;
                case PR_ATTACH_SIZE:
                    size = pRows->aRow[0].lpProps[i].Value.ul;
                    attachmentParam.setSize(size);
                    break;
                case PR_ATTACH_FILENAME: {
                    filename = pRows->aRow[0].lpProps[i].Value.lpszW;                    
                    char* f = toMultibyte(filename.c_str());
                    attachmentParam.setName(StringBuffer(f));
                    delete [] f;
                    break;
                                         }
                case PR_MSG_STATUS:
                    msgstatus = pRows->aRow[0].lpProps[i].Value.ul;                    
                    break;
                case 0x30000003:
                    // pRows->aRow[0].lpProps[ii].Value.ul: sequence number
                    break;
                default:
                    break;
            }    

        }
        FreeProws(pRows);
        
        // if the status is set to MSGSTATUS_REMOTE_DOWNLOAD, this is the one choosen 
        // to be downloaded
        if (!(msgstatus & MSGSTATUS_REMOTE_DOWNLOAD)) {
            continue;
        }
        // MessageBox(NULL, filename.c_str(), filename.c_str(), MB_OK);
        pmsg->OpenAttach(num, NULL, 0, &attach);
        IStream* pStream;
        hr = attach->OpenProperty(PR_ATTACH_DATA_BIN, 0, NULL, ATTACH_BY_VALUE, (LPUNKNOWN *)&pStream);
        EXIT_ON_FAILED(hr);
                
        int m = 0;
        StringBuffer url("");
        do {
            pStream->Read(buffer, 2048, &ulNumChars);
            url += buffer;
            m = m + ulNumChars;

        } while (ulNumChars >= 2048);
        
        pStream->Release();
        url.replaceAll("&amp;", "&"); // remove the &amp; that was inserted somewhere
        attachmentParam.setURL(url);
        ret = true;       
    }
FuncExit:
    // Free each element in the pRows buffer
    FreeProws(pRows);
    RELEASE_OBJ(pAttachTable);
    LOG.debug("Exiting from GetAttachments function");
    return ret;

}


bool CAdviseSink::checkNotifyOnInbox(const wstring& folderKey) 
{
    return checkNotify(folderKey, ROLE_INBOX);
}

bool CAdviseSink::checkNotifyOnOutbox(const wstring& folderKey) 
{
    return checkNotify(folderKey, ROLE_OUTBOX);
}

bool CAdviseSink::checkNotifyOnTrash(const wstring& folderKey) 
{
    return checkNotify(folderKey, ROLE_TRASH);
}


bool CAdviseSink::checkNotify(const wstring& folderKey, const StringBuffer& folderRole) 
{
    bool notify = false;

    // refersh client mail sync source 
    mssc = getRegConfig()->getMailSyncSourceConfig(true);

    if (mssc == NULL) {
        LOG.error("error refreshing mail sync source config");

        return notify;
    }

    // get defined mail accounts
    const ArrayList& mailAccounts = mssc->getMailAccounts();
    int accountNum = 0;
    
    if ((accountNum = mailAccounts.size()) == 0) {
        LOG.info("no mail accounts defined");

        return notify;
    }

    for (int i = 0; i < accountNum; i++) {
        const WCHAR* accountId = ((MailAccount*)mailAccounts[i])->getID();
        if (accountId == NULL) {
            LOG.error("invalid account ID");
            continue;
        }

        if (checkAccountFolderById(accountId, folderKey, folderRole)) {
            notify = true;
            break;
        }
    }

    return notify;
}

bool CAdviseSink::checkAccountFolderById(const wchar_t* accountId, 
                                         const wstring& folderKey, 
                                         const StringBuffer& folderRole)
{
    HRESULT             hr;
    IMAPISession *      pSession        =  NULL;
    ULONG               returned        =  0;
    LPMDB               lpMapiStore     = NULL;
    ULONG               messageStatus   = (ULONG) 0x00000000;
    SBinary             accountEntry;
    bool                match           = false;
    SPropValue*         msgprops        = NULL;
    ULONG               cmsgValue       =  0;
    LPSPropValue        tmp             = NULL;
    wstring             targetFolderKey;

    accountEntry = msgMan->createIdFromKey(accountId);

    if (accountEntry.cb == 0) {
        LOG.error("invalid ID from account entry");
        return false;
    }

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);

    if (hr != S_OK) {
        LOG.error("MAPI logon error");

        return false;
    }

    hr = pSession->OpenEntry(accountEntry.cb,
            (ENTRYID*)accountEntry.lpb,
            NULL,
            MAPI_BEST_ACCESS,
            &returned,
            (IUnknown **) &lpMapiStore);

    if (!lpMapiStore) {
        LOG.error("error opening MAPI message store");
        
        goto FuncExit;
    }

    ULONG folderRoleTags[2];
    folderRoleTags[0] = 1;

    if (folderRole == ROLE_INBOX) {
        folderRoleTags[1] = PR_CE_IPM_INBOX_ENTRYID; 
    } else if (folderRole == ROLE_OUTBOX) {
        folderRoleTags[1] = PR_IPM_OUTBOX_ENTRYID;
    } else if (folderRole == ROLE_TRASH) {
        folderRoleTags[1] = PR_IPM_WASTEBASKET_ENTRYID;
    } else {
        LOG.error("invalid folder role: %s", folderRole.c_str());
        goto FuncExit;
    }

    hr = lpMapiStore->GetProps ((LPSPropTagArray)&folderRoleTags, MAPI_UNICODE, &cmsgValue,
            &msgprops);

    if ( !msgprops ) {
        LOG.error("error getting message props");

        goto FuncExit;
    }

    // get sbinary for account inbox folder
    targetFolderKey = MessagingManager::createKeyFromId(msgprops[0].Value.bin);

    // check if the folder entries match
    if (targetFolderKey == folderKey) {
        match = true;    
    } 

FuncExit:

    if (msgprops) {
        MAPIFreeBuffer(msgprops);
    }
    if (lpMapiStore) {
        lpMapiStore->Release();
    }
    if (pSession) {
        pSession->Release();
    }

    return match;
}


HRESULT CAdviseSink::updateFlagMail(SBinary sbEntry, int flagToUpdate) {

    HRESULT           hr;
    ULONG             returned    =  0;
    ULONG             messageStatus = (ULONG) 0x00000000;

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &globalpSession);

    hr = globalpSession->OpenEntry(sbEntry.cb,
        (ENTRYID*)sbEntry.lpb,
        NULL,
        MAPI_BEST_ACCESS,
        &returned,
        (IUnknown **) &globalpmsg);

    if(!globalpmsg){
        LOG.error("updateFlagMail: pmsg is null.");
        goto FuncExit;
    }

    SPropValue* msgprops = NULL;
    ULONG    cmsgValue =  0;

    hr = globalpmsg->GetProps ((LPSPropTagArray)&Columns, MAPI_UNICODE, &cmsgValue,
       &msgprops);

    if (msgprops[ePR_MSG_STATUS].ulPropTag == PR_MSG_STATUS) {
        messageStatus = msgprops[ePR_MSG_STATUS].Value.ul;
    }

    SPropValue rgprops[1] = {0};
    ULONG    cProps   = 0;

    rgprops[cProps].ulPropTag = PR_MSG_STATUS;
    if (flagToUpdate == 1) {
        messageStatus = messageStatus ^ MSGSTATUS_REMOTE_DOWNLOAD;
    }
    if (flagToUpdate == 2) {
        // not used here otherwise the MessageUI is not working fine. It is frozen!!!!
        messageStatus = messageStatus ^ MSGSTATUS_REMOTE_DOWNLOAD_ATTACH;    
    }
    rgprops[cProps].Value.ul = messageStatus;
    ++cProps;

    hr = globalpmsg->SetProps(cProps, rgprops, NULL);

FuncExit:

    if (msgprops) {
        MAPIFreeBuffer(msgprops);
    }

    if (globalpmsg) {
        globalpmsg->Release();
        globalpmsg = NULL;
    }

    if (globalpSession) {
        globalpSession->Release();
        globalpSession = NULL;
    }

    return NULL;
}



/**
* Check if the attachment mutex exists. if 
*/
bool checkAttachmentMutex() {
    
    HANDLE hMutex = NULL;      
    bool ret = false;

    // Create mutex here
    hMutex = CreateMutex(NULL, TRUE, TEXT("FunSyncInProgress") );
    
    if(hMutex){
        DWORD err = GetLastError();
        if(err == ERROR_ALREADY_EXISTS){
            ret = true;
        }
        CloseHandle(hMutex);
    }
        
    return ret;
        
}

STDMETHODIMP_(ULONG) CAdviseSink::OnNotify (ULONG cNotify, LPNOTIFICATION lpNotifications) {

    HRESULT    hRes = S_OK;
    int flagUpdate = 0;

    if (checkMailSyncInProgress()) {
        return 0;
    }

    try {
        for (ULONG i=0 ; i<cNotify ; i++)
        {
            hRes = S_OK;

            if ((fnevObjectModified == lpNotifications[i].ulEventType) &&
                (lpNotifications[i].info.obj.ulObjType == MAPI_MESSAGE)) 
            {
                bool status = false;
                bool readFlag = false;

                // the status is changed. TAKE CARE OF the lpPropTagArray[0].cValues
                for (unsigned int k = 0; k < lpNotifications[i].info.obj.lpPropTagArray[0].cValues; k++) {
                    if (lpNotifications[i].info.obj.lpPropTagArray[0].aulPropTag[0] == PR_MSG_STATUS) {//PR_MESSAGE_FLAGS
                        status = true;
                    }
                    if (lpNotifications[i].info.obj.lpPropTagArray[0].aulPropTag[0] == PR_MESSAGE_FLAGS) {
                        readFlag = true;
                    }
                }
                
                if (readFlag){
                    SBinary sbEntry;
                    SBinary sbParentEntry;

                    sbParentEntry.cb = lpNotifications[i].info.obj.cbParentID;
                    sbParentEntry.lpb = (LPBYTE)lpNotifications[i].info.obj.lpParentID;
                    sbEntry.cb = lpNotifications[i].info.statobj.cbEntryID;
                    sbEntry.lpb = (LPBYTE)lpNotifications[i].info.statobj.lpEntryID;
                   
                    wstring parentEntryKey = MessagingManager::createKeyFromId(sbParentEntry);
                    
                    // check if the modify notification is for a mail in one of our
                    // inbox accounts and register into messaging cache, otherwise 
                    // silently discard it
                    if (checkNotifyOnInbox(parentEntryKey)) {
                        isMarkedChangedFlag(sbEntry, parentEntryKey, ROLE_INBOX);    
                    } 
                }

                if (status) {
                    
                    SBinary sbEntry;
                    SBinary sbParentEntry;
                    
                    sbParentEntry.cb = lpNotifications[i].info.obj.cbParentID;
                    sbParentEntry.lpb = (LPBYTE)lpNotifications[i].info.obj.lpParentID;
                    
                    sbEntry.cb = lpNotifications[i].info.statobj.cbEntryID;
                    sbEntry.lpb = (LPBYTE)lpNotifications[i].info.statobj.lpEntryID;

                    wstring parentEntryKey = MessagingManager::createKeyFromId(sbParentEntry);

   
                    if (checkNotifyOnInbox(parentEntryKey)) {
                        if ((flagUpdate = isMarkedToDownload(sbEntry)) > 0) {
                            
                            wstring s = MessagingManager::createKeyFromId(sbEntry);                            
                            wchar_t command[1024];    

                            if (flagUpdate == 1) {
                                /*
                                * It we are not using the MEA server, the id must be in the old format
                                * I/<message_id>
                                */
                                if (getRegConfig()->getServerConfig().getMultipleEmailAccount() != 1) {
                                    s.insert(0, TEXT("I/"));                                
                                }
                                wsprintf(command, TEXT("inclusive mail,%s"), s.c_str());
                                int pid = startProgram(SYNCAPP, command);	
                                updateFlagMail(sbEntry, flagUpdate);

                            } else if (flagUpdate == 2) {
                                
                                bool ret = resetAttachmentFlag(sbEntry);
                                if (checkAttachmentMutex() == false) {                                                       
                                    wsprintf(command, TEXT("attachment %s,%lu,%S,%lu"), s.c_str(),
                                                      attachmentParam.getNumber(), 
                                                      attachmentParam.getURL().c_str(),
                                                      attachmentParam.getSize());
                                    int pid = startProgram(SYNCAPP, command);
                                    if (pid) {
                                        Sleep(1500); // sleep some little time to wait the actions in the 
                                                    // startsync process
                                    }
                                }                                                                                         
                            }                            
                        }
                    }
                }
            }

            if (fnevNewMail == lpNotifications[i].ulEventType) {
                LOG.debug("catching fnevNewMail event notification");
            }
            if (fnevObjectDeleted == lpNotifications[i  ].ulEventType) {}
            if (fnevObjectCopied == lpNotifications[i].ulEventType){}
            if (fnevObjectMoved == lpNotifications[i].ulEventType) {
                if (lpNotifications[i].info.obj.ulObjType == MAPI_MESSAGE) {
                    SBinary sbEntry;
                    SBinary sbParentEntry;
               
                    sbParentEntry.cb = lpNotifications[i].info.obj.cbParentID;
                    sbParentEntry.lpb = (LPBYTE)lpNotifications[i].info.obj.lpParentID;
                    sbEntry.cb = lpNotifications[i].info.statobj.cbEntryID;
                    sbEntry.lpb = (LPBYTE)lpNotifications[i].info.statobj.lpEntryID;
               
                    wstring parentEntryKey = MessagingManager::createKeyFromId(sbParentEntry);

                    // check if a new mail has been sent 
                    if (checkNotifyOnOutbox(parentEntryKey)) {
                        isNewMailSent(sbEntry, parentEntryKey);
                    } else 
                    // check if delete notification is for a message in one of
                    // the funambol defined mail accounts
                    if (checkNotifyOnTrash(parentEntryKey)) {
                        isMailDeleted(sbEntry, parentEntryKey); 
                    } 
                }
            } 

            if (fnevObjectCreated == lpNotifications[i].ulEventType) {}

            if (fnevTableModified == lpNotifications[i].ulEventType) {
                switch(lpNotifications[i].info.tab.ulTableEvent)
                {
                case(TABLE_ERROR):
                case(TABLE_CHANGED):
                case(TABLE_RELOAD):
                    LOG.debug("TABLE_RELOAD MAIL advise");
                    break;
                case(TABLE_ROW_ADDED):
                    LOG.debug("TABLE_ROW_ADDED MAIL advise");
                    break;
                case(TABLE_ROW_DELETED):
                    LOG.debug("TABLE_ROW_DELETED MAIL advise");
                    break;
                case(TABLE_ROW_MODIFIED):
                    LOG.debug("TABLE_ROW_MODIFIED MAIL advise");
                    break;
                }
            }
        }
        if (globalpmsg) {
            globalpmsg->Release();
            globalpmsg = NULL;
        }
        if (globalpSession) {
            globalpSession->Release();
            globalpSession = NULL;
        }

    }
    catch(...) {
        LOG.error("Advisesink error");
    }
    HWND wnd = GetForegroundWindow();
    ::UpdateWindow(wnd);    
    return 0;
}


bool CAdviseSink::resetAttachmentFlag(SBinary sbEntry) {
    
    HRESULT           hr;
    IMAPISession *    pSession      =  NULL;
    ULONG             returned      =  0;
    IMessage *        pmsg          =  NULL;
    ULONG             messageStatus = (ULONG) 0x00000000;
    bool              ret           = false;

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);

    hr = pSession->OpenEntry(sbEntry.cb,
        (ENTRYID*)sbEntry.lpb,
        NULL,
        MAPI_BEST_ACCESS,
        &returned,
        (IUnknown **) &pmsg);

    if(!pmsg){
        LOG.error("startDownloadAttachment: pmsg is null.");
        goto exit;
    }
    
    IAttach*    attach = NULL; 
    hr = pmsg->OpenAttach(attachmentParam.getNumber(), NULL, 0, &attach);
    if (hr != S_OK) {
        LOG.error("startDownloadAttachment: OpenAttach failed.");
        goto exit;
    } 
                   
    SPropValue  prop[1];               
    memset(prop, 0, sizeof(prop));    
    prop[0].ulPropTag    = PR_MSG_STATUS;
    prop[0].Value.ul     = MSGSTATUS_HEADERONLY;      
    hr = attach->SetProps(1, prop, NULL);
    attach->Release();
    ret = true;
exit:
    
    if (pmsg) {
        pmsg->Release();
        pmsg = NULL;
    }

    if (pSession) {
        pSession->Release();
        pSession = NULL;
    }

    return ret;    
}

bool CAdviseSink::checkMailSyncInProgress() 
{
  bool ret = false;
  HANDLE hMutex = CreateMutex(NULL, TRUE, TEXT("FunMailSyncInProgress") );

    if (hMutex){
        if(GetLastError() == ERROR_ALREADY_EXISTS){
            ret = true;
        }
    
        CloseHandle(hMutex);
    }

    return ret;
}
