/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "base/Log.h"
#include "base/util/utils.h"
#include "PushManager.h"
#include "pim/ClientSettings.h"
#include "http/GPRSConnection.h"
#include "pim/Utils.h"
#include "notify/s4n_service.h"
#include "processUtils.h"
#include "pim/maincpp.h"

// In case of need, we may move these defines in customization.h
#define FUNOPT_ENABLE_C2S_PUSH      1
#define FUNOPT_ENABLE_S2C_PUSH      1
#define FUNOPT_ENABLE_SMS_PUSH      1


static wstring getCompleteAppName(const wchar_t* appName);
static void createLink();
static void deleteLink();

#define FUNAMBOL_LNK TEXT("\\") TEXT(APP_NAME) TEXT(".lnk")

// Init static pointer.
PushManager* PushManager::pinstance = NULL;

// method used to get the singleton instance
PushManager* getPushManager() {
    return PushManager::getInstance();
}

/**
 * Method to create the sole instance of PushManager
 */
PushManager* PushManager::getInstance() {

    if (pinstance == NULL) {
        pinstance = new PushManager;
    }
    return pinstance;
}


PushManager::PushManager() {
    
}

PushManager::~PushManager() {
       
}

bool PushManager::registerOnAll() {
    
    if (FUNOPT_ENABLE_SMS_PUSH) {
        if (registerOnSMS()) {
            LOG.info("PushManager: registered the SMS push");
        } else {
            LOG.error("PushManager: error registering the SMS push");
        }
    }

    if (FUNOPT_ENABLE_S2C_PUSH) {
        if (registerOnHTTP()) {
            LOG.info("PushManager: registered the HTTP (CTP, STP) push");
        } else {
            LOG.error("PushManager: error registering the HTTP (CTP, STP) push");
        }
    }
    
    if (FUNOPT_ENABLE_C2S_PUSH) {
        if (registerOnCP()) {
             LOG.info("PushManager: registered the CP push");
        } else {
            LOG.error("PushManager: error registering the CP push");
        }
    }
    return true;
}

bool PushManager::registerOnSMS() {
    
    bool ret = true;

    wstring path = getCompleteAppName(SYNCAPP); // /Program files/Funambol/startsync.exe
    
    HRESULT hr = PushRouter_RegisterClient(ContentType, AppId, path.c_str(), Params);
    if (hr == S_OK) {
        LOG.info("PushManager: Wap push client registered successfully");
    } else if (hr == PUSHRTR_E_ALREADYREGISTERED) {
        LOG.debug("Wap push client already registered");
    } else {
        LOG.error("Error to register wap push client");
        ret = false;
    }
    return ret;
}

/**
* Used to register the service without create the link for the startup
*/ 
bool PushManager::registerOnHTTPNoLink() {

    int ret = true;        

    ret = registerService();
    if (ret != 0) {
        LOG.info("PushManager: error registering http: %i", ret);
    }

    if (!EstablishConnection()) {
        LOG.info("PushManager: not possible to establish the connection");
    }
        
    return ret == 0 ? true : false;
}

/**
* register the http push (ctp/stp)
*/
bool PushManager::registerOnHTTP() {

    bool ret = registerOnHTTPNoLink();           
    
    createLink();

    return ret;
}

// to be implemented
bool PushManager::registerOnCP() {
    
    if (registerCP()) {
        LOG.info("PushManager: registerCP");
    } else {
        LOG.error("PushManager: error registerCP");
    }
    return registerCP();
}

bool PushManager::deregisterOnAll() {
    
    if (FUNOPT_ENABLE_SMS_PUSH) {
        if (deregisterOnSMS()) {
            LOG.info("PushManager: deregisterOnSMS the SMS push");
        } else {
            LOG.error("PushManager: error deregistering the SMS push");
        }
    }
    
    if (FUNOPT_ENABLE_S2C_PUSH) {
        if (deregisterOnHTTP()) {
            LOG.info("PushManager: deregisterOnHTTP the HTTP (CTP, STP) push");
        } else {
            LOG.error("PushManager: error deregistering the HTTP (CTP, STP) push");
        }
    }
    
    if (FUNOPT_ENABLE_C2S_PUSH) {
        if (deregisterOnCP()) {
            LOG.info("PushManager: deregisterOnCP push");
        } else {
            LOG.error("PushManager: error deregistering the CP push");
        }  
    }
    return true;
}

bool PushManager::deregisterOnSMS() {
   
    bool ret = true;
    HRESULT hr = PushRouter_UnRegisterClient(ContentType, AppId);
    
    if (hr == S_OK) {
        LOG.debug("PushManager: Wap push client unregistered successfully");
    } else if (hr == PUSHRTR_E_NOTFOUND) {
        LOG.debug("PushManager: Wap push client not found. Arleady unregistered");
    } else {
        ret = false;
        LOG.error("PushManager: Error to unregister wap push client");
    }
    return ret;
}

bool PushManager::deregisterOnHTTP() {
    
    int ret = true;    
    
    deleteLink();
    
    ret = deregisterService();
    if (ret != 0) {
        LOG.info("PushManager: deregisterService error");
    }
   
    return ret == 0 ? true : false;
}

bool PushManager::deregisterOnCP() {
    return deregisterCP();           
}


static wstring getCompleteAppName(const wchar_t* appName) {
    
    ClientSettings* cs = getRegConfig();
    wchar_t* t = toWideChar(cs->getAppPath().c_str());
    wstring s = t;
    s += TEXT("\\");
    s += appName;    
    
    delete [] t;
    return s;
}

static void createLink() {
    wchar_t pathRetrieved[512];
    SHGetSpecialFolderPath(NULL, pathRetrieved, CSIDL_STARTUP, 0);    
    wcscat(pathRetrieved, FUNAMBOL_LNK);

    wstring name = getCompleteAppName(SYNCAPP);

    wstring s;
    s.append(TEXT("\""));
    s.append(name.c_str());
    s.append(TEXT("\" registerByLink"));
    LOG.debug("PushManager: creating the startup link %S", s.c_str());

    if (SHCreateShortcut(pathRetrieved, (wchar_t*)s.c_str()) == FALSE) {
        LOG.error("PushManager: error creating the link");
    }
}

static void deleteLink() {

    wchar_t pathRetrieved[512];
    SHGetSpecialFolderPath(NULL, pathRetrieved, CSIDL_STARTUP, 0);
    wcscat(pathRetrieved, FUNAMBOL_LNK);
    if (DeleteFile(pathRetrieved) == false) {
        LOG.error("PushManager: error deleting the link");
    }
}
